@@ -1,14 +1,15 @@
 /*
- * Copyright (C) 2001 - 2002 Carlos Perell Marn <carlos@gnome-db.org>
+ * Copyright (C) 2001 - 2002 Carlos Perelló Marín <carlos@gnome-db.org>
  * Copyright (C) 2002 - 2003 Gonzalo Paniagua Javier <gonzalo@src.gnome.org>
  * Copyright (C) 2002 - 2005 Rodrigo Moya <rodrigo@gnome-db.org>
  * Copyright (C) 2005 Denis Fortin <denis.fortin@free.fr>
- * Copyright (C) 2005 - 2011 Vivien Malerba <malerba@gnome-db.org>
- * Copyright (C) 2005 lvaro Pea <alvaropg@telefonica.net>
+ * Copyright (C) 2005 - 2013 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2005 Álvaro Peña <alvaropg@telefonica.net>
  * Copyright (C) 2006 - 2008 Murray Cumming <murrayc@murrayc.com>
  * Copyright (C) 2007 Armin Burgmeier <arminb@src.gnome.org>
  * Copyright (C) 2009 Bas Driessen <bas.driessen@xobas.com>
  * Copyright (C) 2010 David King <davidk@openismus.com>
+ * Copyright (C) 2011 Marco Ciampa <ciampix@libero.it>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -37,6 +38,9 @@
 #include <libgda/gda-util.h>
 #include <libgda/gda-connection-private.h>
 
+#include "virtual/gda-vconnection-data-model.h"
+#include "virtual/gda-vconnection-data-model-private.h"
+
 #define _GDA_PSTMT(x) ((GdaPStmt*)(x))
 
 static void gda_sqlite_recordset_class_init (GdaSqliteRecordsetClass *klass);
@@ -53,6 +57,8 @@ static gboolean gda_sqlite_recordset_fetch_next (GdaDataSelect *model, GdaRow **
 
 static GdaRow *fetch_next_sqlite_row (GdaSqliteRecordset *model, gboolean do_store, GError **error);
 
+static void virt_cnc_set_working_obj (GdaConnection *cnc, GdaSqliteRecordset *model);
+
 struct _GdaSqliteRecordsetPrivate {
 	gboolean      empty_forced;
 	gint          next_row_num;
@@ -105,7 +111,9 @@ gda_sqlite_recordset_dispose (GObject *object)
 		GdaSqlitePStmt *ps;
 		ps = GDA_SQLITE_PSTMT (GDA_DATA_SELECT (object)->prep_stmt);
 		ps->stmt_used = FALSE;
+		virt_cnc_set_working_obj (gda_data_select_get_connection ((GdaDataSelect*) recset), recset);
 		SQLITE3_CALL (sqlite3_reset) (ps->sqlite_stmt);
+		virt_cnc_set_working_obj (gda_data_select_get_connection ((GdaDataSelect*) recset), NULL);
 
 		if (recset->priv->tmp_row)
 			g_object_unref (recset->priv->tmp_row);
@@ -126,7 +134,7 @@ _gda_sqlite_recordset_get_type (void)
 	static GType type = 0;
 
 	if (G_UNLIKELY (type == 0)) {
-		static GStaticMutex registering = G_STATIC_MUTEX_INIT;
+		static GMutex registering;
 		static const GTypeInfo info = {
 			sizeof (GdaSqliteRecordsetClass),
 			(GBaseInitFunc) NULL,
@@ -139,10 +147,10 @@ _gda_sqlite_recordset_get_type (void)
 			(GInstanceInitFunc) gda_sqlite_recordset_init,
 			0
 		};
-		g_static_mutex_lock (&registering);
+		g_mutex_lock (&registering);
 		if (type == 0)
 			type = g_type_register_static (GDA_TYPE_DATA_SELECT, CLASS_PREFIX "Recordset", &info, 0);
-		g_static_mutex_unlock (&registering);
+		g_mutex_unlock (&registering);
 	}
 
 	return type;
@@ -201,7 +209,7 @@ read_rows_to_init_col_types (GdaSqliteRecordset *model)
  */
 GdaDataModel *
 _gda_sqlite_recordset_new (GdaConnection *cnc, GdaSqlitePStmt *ps, GdaSet *exec_params,
-			  GdaDataModelAccessFlags flags, GType *col_types, gboolean force_empty)
+			   GdaDataModelAccessFlags flags, GType *col_types, gboolean force_empty)
 {
 	GdaSqliteRecordset *model;
         SqliteConnectionData *cdata;
@@ -283,6 +291,13 @@ _gda_sqlite_recordset_new (GdaConnection *cnc, GdaSqlitePStmt *ps, GdaSet *exec_
 			      "prepared-stmt", ps, "model-usage", rflags, 
 			      "exec-params", exec_params, 
 			      "auto-reset", force_empty, NULL);
+	gboolean is_virt;
+	is_virt = GDA_IS_VCONNECTION_DATA_MODEL (cnc) ? TRUE : FALSE;
+	if (is_virt) {
+		/* steal the lock */
+		_gda_vconnection_change_working_obj ((GdaVconnectionDataModel*) cnc, (GObject*) model);
+		_gda_vconnection_set_working_obj ((GdaVconnectionDataModel*) cnc, NULL);
+	}
 
         /* fill the data model */
         read_rows_to_init_col_types (model);
@@ -318,6 +333,15 @@ fuzzy_get_gtype (SqliteConnectionData *cdata, GdaSqlitePStmt *ps, gint colnum)
 	return gtype;
 }
 
+static void
+virt_cnc_set_working_obj (GdaConnection *cnc, GdaSqliteRecordset *model)
+{
+	gboolean is_virt;
+	is_virt = GDA_IS_VCONNECTION_DATA_MODEL (cnc) ? TRUE : FALSE;
+	if (is_virt)
+		_gda_vconnection_set_working_obj ((GdaVconnectionDataModel*) cnc, (GObject*) model);
+}
+
 static GdaRow *
 fetch_next_sqlite_row (GdaSqliteRecordset *model, gboolean do_store, GError **error)
 {
@@ -332,9 +356,11 @@ fetch_next_sqlite_row (GdaSqliteRecordset *model, gboolean do_store, GError **er
 		return NULL;
 	ps = GDA_SQLITE_PSTMT (GDA_DATA_SELECT (model)->prep_stmt);
 
+	virt_cnc_set_working_obj (gda_data_select_get_connection ((GdaDataSelect*) model), model);
+
 	if (model->priv->empty_forced)
 		rc = SQLITE_DONE;
-	else
+	else		
 		rc = SQLITE3_CALL (sqlite3_step) (ps->sqlite_stmt);
 	switch (rc) {
 	case  SQLITE_ROW: {
@@ -524,8 +550,11 @@ fetch_next_sqlite_row (GdaSqliteRecordset *model, gboolean do_store, GError **er
 							     (gchar *) SQLITE3_CALL (sqlite3_column_text) (ps->sqlite_stmt, real_col));
 						gda_row_invalidate_value_e (prow, value, lerror);
 					}
-					else
+					else {
+						if (timegda.timezone == GDA_TIMEZONE_INVALID)
+							timegda.timezone = 0; /* set to GMT */
 						gda_value_set_time (value, &timegda);
+					}
 				}
 				else if (type == GDA_TYPE_TIMESTAMP) {
 					GdaTimestamp timestamp;
@@ -539,8 +568,11 @@ fetch_next_sqlite_row (GdaSqliteRecordset *model, gboolean do_store, GError **er
 							     (gchar *) SQLITE3_CALL (sqlite3_column_text) (ps->sqlite_stmt, real_col));
 						gda_row_invalidate_value_e (prow, value, lerror);
 					}
-					else
+					else {
+						if (timestamp.timezone == GDA_TIMEZONE_INVALID)
+							timestamp.timezone = 0; /* set to GMT */
 						gda_value_set_timestamp (value, &timestamp);
+					}
 				}
 				else if (type == G_TYPE_CHAR) {
 					gint64 i;
@@ -553,11 +585,7 @@ fetch_next_sqlite_row (GdaSqliteRecordset *model, gboolean do_store, GError **er
 						gda_row_invalidate_value_e (prow, value, lerror);
 					}
 					else
-#if GLIB_CHECK_VERSION(2,31,7)
 						g_value_set_schar (value, (gchar) i);
-#else
-						g_value_set_char (value, (gchar) i);
-#endif
 				}
 				else if (type == G_TYPE_UCHAR) {
 					gint64 i;
@@ -647,6 +675,9 @@ fetch_next_sqlite_row (GdaSqliteRecordset *model, gboolean do_store, GError **er
 		break;
 	}
 	}
+
+	virt_cnc_set_working_obj (gda_data_select_get_connection ((GdaDataSelect*) model), NULL);
+
 	return prow;
 }
 
