@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2007 - 2011 Vivien Malerba <malerba@gnome-db.org>
- * Copyright (C) 2008 Murray Cumming <murrayc@murrayc.com>
+ * Copyright (C) 2007 - 2013 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2008 - 2011 Murray Cumming <murrayc@murrayc.com>
  * Copyright (C) 2009 Bas Driessen <bas.driessen@xobas.com>
  * Copyright (C) 2010 David King <davidk@openismus.com>
  *
@@ -30,7 +30,10 @@
 #include "../gda-sqlite.h"
 
 struct _GdaVconnectionDataModelPrivate {
-	GSList *table_data_list; /* list of GdaVConnectionTableData structures */
+	GSList       *table_data_list; /* list of GdaVConnectionTableData structures */
+
+	GRecMutex     lock_context;
+	GdaStatement *executed_stmt;
 };
 
 static void gda_vconnection_data_model_class_init (GdaVconnectionDataModelClass *klass);
@@ -75,7 +78,7 @@ static void
 vtable_dropped (GdaVconnectionDataModel *cnc, const gchar *table_name)
 {
 	GdaVConnectionTableData *td;
-	td = gda_vconnection_get_table_data_by_name (cnc, table_name);
+	td = _gda_vconnection_get_table_data_by_name (cnc, table_name);
 	if (td)
 		cnc->priv->table_data_list = g_slist_remove (cnc->priv->table_data_list, td);
 	_gda_connection_signal_meta_table_update ((GdaConnection *)cnc, table_name);
@@ -136,6 +139,7 @@ gda_vconnection_data_model_init (GdaVconnectionDataModel *cnc, G_GNUC_UNUSED Gda
 {
 	cnc->priv = g_new (GdaVconnectionDataModelPrivate, 1);
 	cnc->priv->table_data_list = NULL;
+	g_rec_mutex_init (& (cnc->priv->lock_context));
 
 	g_object_set (G_OBJECT (cnc), "cnc-string", "_IS_VIRTUAL=TRUE", NULL);
 }
@@ -156,6 +160,7 @@ gda_vconnection_data_model_dispose (GObject *object)
 		}
 		gda_connection_close_no_warning ((GdaConnection *) cnc);
 
+		g_rec_mutex_clear (& (cnc->priv->lock_context));
 		g_free (cnc->priv);
 		cnc->priv = NULL;
 	}
@@ -175,7 +180,7 @@ gda_vconnection_data_model_get_type (void)
 	static GType type = 0;
 
 	if (G_UNLIKELY (type == 0)) {
-		static GStaticMutex registering = G_STATIC_MUTEX_INIT;
+		static GMutex registering;
 		if (type == 0) {
 			static GTypeInfo info = {
 				sizeof (GdaVconnectionDataModelClass),
@@ -188,11 +193,11 @@ gda_vconnection_data_model_get_type (void)
 				(GInstanceInitFunc) gda_vconnection_data_model_init,
 				0
 			};
-			
-		g_static_mutex_lock (&registering);
-		if (type == 0)
-			type = g_type_register_static (GDA_TYPE_VIRTUAL_CONNECTION, "GdaVconnectionDataModel", &info, 0);
-		g_static_mutex_unlock (&registering);
+
+			g_mutex_lock (&registering);
+			if (type == 0)
+				type = g_type_register_static (GDA_TYPE_VIRTUAL_CONNECTION, "GdaVconnectionDataModel", &info, 0);
+			g_mutex_unlock (&registering);
 		}
 	}
 
@@ -241,13 +246,13 @@ gboolean
 gda_vconnection_data_model_add_model (GdaVconnectionDataModel *cnc, 
 				      GdaDataModel *model, const gchar *table_name, GError **error)
 {
+	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
 	GdaVconnectionDataModelSpec *spec;
 	gboolean retval;
 
 	spec = g_new0 (GdaVconnectionDataModelSpec, 1);
-	spec->data_model = model;
+	spec->data_model = g_object_ref (model);
 	spec->create_columns_func = create_columns;
-	g_object_ref (model);
 	retval = gda_vconnection_data_model_add (cnc, spec, (GDestroyNotify) spec_destroy_func, table_name, error);
 
 	return retval;
@@ -330,7 +335,7 @@ gda_vconnection_data_model_add (GdaVconnectionDataModel *cnc, GdaVconnectionData
 			     GDA_SERVER_PROVIDER_INTERNAL_ERROR,
 			     "%s", zErrMsg);
 		SQLITE3_CALL (sqlite3_free) (zErrMsg);
-		gda_vconnection_data_model_table_data_free (td);
+		_gda_vconnection_data_model_table_data_free (td);
 		cnc->priv->table_data_list = g_slist_remove (cnc->priv->table_data_list, td);
 		retval = FALSE;
 	}
@@ -377,7 +382,7 @@ get_rid_of_vtable (GdaVconnectionDataModel *cnc, GdaVConnectionTableData *td, gb
 	g_signal_emit (G_OBJECT (cnc), gda_vconnection_data_model_signals[VTABLE_DROPPED], 0,
 		       td->table_name);
 	/*g_print ("Virtual connection: removed table %s (%p)\n", td->table_name, td->spec->data_model);*/
-	gda_vconnection_data_model_table_data_free (td);
+	_gda_vconnection_data_model_table_data_free (td);
 
 	return allok;
 }
@@ -401,7 +406,7 @@ gda_vconnection_data_model_remove (GdaVconnectionDataModel *cnc, const gchar *ta
 	g_return_val_if_fail (GDA_IS_VCONNECTION_DATA_MODEL (cnc), FALSE);
 	g_return_val_if_fail (table_name && *table_name, FALSE);
 
-	td = gda_vconnection_get_table_data_by_name (cnc, table_name);
+	td = _gda_vconnection_get_table_data_by_name (cnc, table_name);
 	if (!td) {
 		g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
 			     GDA_SERVER_PROVIDER_MISUSE_ERROR,
@@ -431,7 +436,7 @@ gda_vconnection_data_model_get (GdaVconnectionDataModel *cnc, const gchar *table
 	g_return_val_if_fail (GDA_IS_VCONNECTION_DATA_MODEL (cnc), NULL);
 	if (!table_name || !(*table_name))
 		return NULL;
-	td = gda_vconnection_get_table_data_by_name (cnc, table_name);
+	td = _gda_vconnection_get_table_data_by_name (cnc, table_name);
 	if (td)
 		return td->spec;
 	else
@@ -458,7 +463,7 @@ gda_vconnection_data_model_get_model (GdaVconnectionDataModel *cnc, const gchar
 	if (!table_name || !(*table_name))
 		return NULL;
 
-	td = gda_vconnection_get_table_data_by_name (cnc, table_name);
+	td = _gda_vconnection_get_table_data_by_name (cnc, table_name);
 	if (td)
 		return td->spec->data_model;
 	else
@@ -484,7 +489,7 @@ gda_vconnection_data_model_get_table_name (GdaVconnectionDataModel *cnc, GdaData
 		return NULL;
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), NULL);
 
-	td = gda_vconnection_get_table_data_by_model (cnc, model);
+	td = _gda_vconnection_get_table_data_by_model (cnc, model);
 	if (td)
 		return td->table_name;
 	else
@@ -526,7 +531,7 @@ gda_vconnection_data_model_foreach (GdaVconnectionDataModel *cnc,
  * private 
  */
 GdaVConnectionTableData *
-gda_vconnection_get_table_data_by_name (GdaVconnectionDataModel *cnc, const gchar *table_name)
+_gda_vconnection_get_table_data_by_name (GdaVconnectionDataModel *cnc, const gchar *table_name)
 {
 	GSList *list;
 	gchar *quoted;
@@ -544,7 +549,7 @@ gda_vconnection_get_table_data_by_name (GdaVconnectionDataModel *cnc, const gcha
 }
 
 GdaVConnectionTableData *
-gda_vconnection_get_table_data_by_unique_name (GdaVconnectionDataModel *cnc, const gchar *unique_name)
+_gda_vconnection_get_table_data_by_unique_name (GdaVconnectionDataModel *cnc, const gchar *unique_name)
 {
 	GSList *list;
 	for (list = cnc->priv->table_data_list; list; list = list->next) {
@@ -555,7 +560,7 @@ gda_vconnection_get_table_data_by_unique_name (GdaVconnectionDataModel *cnc, con
 }
 
 GdaVConnectionTableData *
-gda_vconnection_get_table_data_by_model (GdaVconnectionDataModel *cnc, GdaDataModel *model)
+_gda_vconnection_get_table_data_by_model (GdaVconnectionDataModel *cnc, GdaDataModel *model)
 {
 	GSList *list;
 	for (list = cnc->priv->table_data_list; list; list = list->next) {
@@ -566,7 +571,7 @@ gda_vconnection_get_table_data_by_model (GdaVconnectionDataModel *cnc, GdaDataMo
 }
 
 void
-gda_vconnection_data_model_table_data_free (GdaVConnectionTableData *td)
+_gda_vconnection_data_model_table_data_free (GdaVConnectionTableData *td)
 {
 	ParamType i;
 
@@ -586,5 +591,105 @@ gda_vconnection_data_model_table_data_free (GdaVConnectionTableData *td)
 		if (td->modif_stmt[i])
 			g_object_unref (td->modif_stmt[i]);
 	}
+
+	if (td->context.hash)
+		g_hash_table_destroy (td->context.hash);
 	g_free (td);
 }
+
+static void
+vcontext_object_weak_notify_cb (VContext *context, GObject *old_context_object)
+{
+	g_assert (context);
+	context->context_object = NULL;
+	g_hash_table_remove (context->vtable->context.hash, old_context_object);
+}
+
+static void
+vcontext_free (VContext *context)
+{
+	if (context->context_object)
+		g_object_weak_unref (context->context_object,
+				     (GWeakNotify) vcontext_object_weak_notify_cb, context);
+	if (context->context_data) {
+		g_array_free (context->context_data, TRUE);
+		context->context_data = NULL;
+	}
+	g_free (context);
+#ifdef DEBUG_VCONTEXT
+	g_print ("VCFree %p\n", context);
+#endif
+}
+
+void
+_gda_vconnection_set_working_obj (GdaVconnectionDataModel *cnc, GObject *obj)
+{
+	GSList *list;
+	if (obj) {
+		g_rec_mutex_lock (& (cnc->priv->lock_context));
+		for (list = cnc->priv->table_data_list; list; list = list->next) {
+			GdaVConnectionTableData *td = (GdaVConnectionTableData*) list->data;
+			VContext *vc = NULL;
+			
+			g_assert (!td->context.current_vcontext);
+			td->context.mutex = &(cnc->priv->lock_context);
+			if (! td->context.hash)
+				td->context.hash = g_hash_table_new_full (g_direct_hash, g_direct_equal,
+									  NULL, (GDestroyNotify) vcontext_free);
+			else
+				vc = g_hash_table_lookup (td->context.hash, obj);
+
+			if (! vc) {
+				vc = g_new0 (VContext, 1);
+				vc->context_object = obj;
+				vc->context_data = g_array_new (FALSE, FALSE,
+								sizeof (VirtualFilteredData*));
+				g_array_set_clear_func (vc->context_data, (GDestroyNotify) _gda_vconnection_virtual_filtered_data_unref);
+				vc->vtable = td;
+				g_object_weak_ref (obj, (GWeakNotify) vcontext_object_weak_notify_cb, vc);
+				g_hash_table_insert (td->context.hash, obj, vc);
+#ifdef DEBUG_VCONTEXT
+				g_print ("VCNew %p\n", vc);
+#endif
+			}
+			td->context.current_vcontext = vc;
+		}
+	}
+	else {
+		for (list = cnc->priv->table_data_list; list; list = list->next) {
+			GdaVConnectionTableData *td = (GdaVConnectionTableData*) list->data;
+			/* REM: td->context.current_vcontext may already be NULL in case
+			 * an exception already occurred */
+			td->context.current_vcontext = NULL;
+		}
+		g_rec_mutex_unlock (& (cnc->priv->lock_context));
+	}
+}
+
+void
+_gda_vconnection_change_working_obj (GdaVconnectionDataModel *cnc, GObject *obj)
+{
+	GSList *list;
+
+	for (list = cnc->priv->table_data_list; list; list = list->next) {
+		GdaVConnectionTableData *td = (GdaVConnectionTableData*) list->data;
+		if (!td->context.hash)
+			continue;
+
+		g_assert (td->context.current_vcontext);
+
+		VContext *ovc, *nvc;
+		ovc = td->context.current_vcontext;
+		nvc = g_new0 (VContext, 1);
+		nvc->context_object = obj;
+		nvc->vtable = ovc->vtable;
+		nvc->context_data = ovc->context_data;
+		ovc->context_data = NULL;
+		g_object_weak_ref (obj, (GWeakNotify) vcontext_object_weak_notify_cb, nvc);
+		g_hash_table_insert (td->context.hash, obj, nvc);
+#ifdef DEBUG_VCONTEXT
+		g_print ("VCNew %p\n", nvc);
+#endif
+		g_hash_table_remove (td->context.hash, ovc->context_object);
+	}
+}
