@@ -1,7 +1,8 @@
 /*
- * Copyright (C) 2008 - 2009 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2008 - 2013 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2009 Bas Driessen <bas.driessen@xobas.com>
  * Copyright (C) 2010 David King <davidk@openismus.com>
+ * Copyright (C) 2011 Murray Cumming <murrayc@murrayc.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -117,7 +118,7 @@ gda_jdbc_recordset_get_type (void)
 	static GType type = 0;
 
 	if (G_UNLIKELY (type == 0)) {
-		static GStaticMutex registering = G_STATIC_MUTEX_INIT;
+		static GMutex registering;
 		static const GTypeInfo info = {
 			sizeof (GdaJdbcRecordsetClass),
 			(GBaseInitFunc) NULL,
@@ -130,16 +131,18 @@ gda_jdbc_recordset_get_type (void)
 			(GInstanceInitFunc) gda_jdbc_recordset_init,
 			0
 		};
-		g_static_mutex_lock (&registering);
+		g_mutex_lock (&registering);
 		if (type == 0)
 			type = g_type_register_static (GDA_TYPE_DATA_SELECT, "GdaJdbcRecordset", &info, 0);
-		g_static_mutex_unlock (&registering);
+		g_mutex_unlock (&registering);
 	}
 
 	return type;
 }
 
-/* Same as GdaJValue::jdbc_type_to_g_type */
+/* Same as GdaJValue::jdbc_type_to_g_type
+ * See http://docs.oracle.com/javase/6/docs/api/constant-values.html#java.sql.Types.ARRAY for reference
+ */
 static GType
 jdbc_type_to_g_type (gint jdbc_type)
 {
@@ -176,22 +179,35 @@ jdbc_type_to_g_type (gint jdbc_type)
  	case 4: /* INTEGER */
 		return G_TYPE_INT;
  	case 2000: /* JAVA_OBJECT */
+		return GDA_TYPE_BINARY;
+	case -16: /* LONGNVARCHAR */
+		return G_TYPE_STRING;
  	case -4: /* LONGVARBINARY */
 		return GDA_TYPE_BINARY;
  	case -1: /* LONGVARCHAR */
 		return G_TYPE_STRING;
+	case -15: /* NCHAR */
+		return G_TYPE_STRING;
+	case 2011: /* NCLOB */
+		return GDA_TYPE_BINARY;
  	case 0: /* NULL */
 		return GDA_TYPE_NULL;
 	case 2: /* NUMERIC */
 		return GDA_TYPE_NUMERIC;
+	case -9: /* NVARCHAR */
+		return G_TYPE_STRING;
 	case 1111: /* OTHER */
 		return GDA_TYPE_BINARY;
  	case 7: /* REAL */
 		return G_TYPE_FLOAT;
  	case 2006: /* REF */
 		return GDA_TYPE_BINARY;
+	case -8: /* ROWID */
+		return G_TYPE_STRING;
  	case 5: /* SMALLINT */
 		return GDA_TYPE_SHORT;
+	case 2009: /* SQLXML */
+		return G_TYPE_STRING;
  	case 2002: /* STRUCT */
 		return GDA_TYPE_BINARY;
  	case 92: /* TIME */
@@ -231,7 +247,7 @@ gda_jdbc_recordset_new (GdaConnection *cnc, GdaJdbcPStmt *ps, GdaSet *exec_param
 	else
 		ps = gda_jdbc_pstmt_new (NULL);
 
-	cdata = (JdbcConnectionData*) gda_connection_internal_get_provider_data_error (cnc, error);
+	cdata = (JdbcConnectionData*) gda_connection_internal_get_provider_data (cnc);
 	if (!cdata)
 		return NULL;
 
@@ -367,9 +383,9 @@ gda_jdbc_recordset_new (GdaConnection *cnc, GdaJdbcPStmt *ps, GdaSet *exec_param
 		gda_value_free (rs_value);
 		return NULL;
 	}
-	
+
 	jexec_res = jni_wrapper_method_call (jenv, GdaJResultSet__declareColumnTypes,
-					     rs_value, &error_code, &sql_state, &error, (jlong)GPOINTER_TO_INT(cnc), jtypes);
+					     rs_value, &error_code, &sql_state, &error, jni_cpointer_to_jlong (cnc), jtypes);
 	(*jenv)->DeleteLocalRef (jenv, jtypes);
 	g_free (ctypes);
 	
@@ -414,7 +430,7 @@ fetch_next_jdbc_row (GdaJdbcRecordset *model, JNIEnv *jenv, gboolean do_store, G
 
 	jexec_res = jni_wrapper_method_call (jenv, GdaJResultSet__fillNextRow,
 					     model->priv->rs_value, &error_code, &sql_state, &lerror,
-					     (jlong)GPOINTER_TO_INT(prow));
+					     jni_cpointer_to_jlong (prow));
 	if (!jexec_res) {
 		if (error && lerror)
 			*error = g_error_copy (lerror);
