@@ -1,6 +1,7 @@
 /*
  * Copyright (C) 2009 - 2011 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2010 David King <davidk@openismus.com>
+ * Copyright (C) 2011 Murray Cumming <murrayc@murrayc.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,7 +26,7 @@
 #include <libgda-ui/gdaui-tree-store.h>
 #include "../dnd.h"
 #include "../support.h"
-#include "../cc-gray-bar.h"
+#include "../gdaui-bar.h"
 #include "marshal.h"
 #include <gdk/gdkkeysyms.h>
 #include <libgda-ui/internal/popup-container.h>
@@ -235,8 +236,8 @@ update_display (ObjectsCloud *cloud)
 	g_slist_free (dbo_list);
 
  out:
-	if (default_sd)
-		schemas = g_slist_prepend (schemas, default_sd);
+	/* default_sd can't be NULL here */
+	schemas = g_slist_prepend (schemas, default_sd);
 
 	/* get rid of the SchemaData structures */
 	for (list = schemas; list; list = list->next) {
